@@ -3,18 +3,24 @@
 #
 # This file is part of the tsfpga project, a project platform for modern FPGA development.
 # https://tsfpga.com
-# https://gitlab.com/tsfpga/tsfpga
+# https://github.com/tsfpga/tsfpga
 # --------------------------------------------------------------------------------------------------
 
 """
 Common functions and definitions in the example build environment.
 """
 
+# Standard libraries
+from typing import TYPE_CHECKING, Optional
+
+if TYPE_CHECKING:
+    from tsfpga.module_list import ModuleList
+
 # Import before others since it modifies PYTHONPATH. pylint: disable=unused-import
 import tsfpga.examples.example_pythonpath  # noqa: F401
 
 # Third party libraries
-from hdl_registers.register_list import Register
+from hdl_registers.register import Register
 
 # First party libraries
 import tsfpga
@@ -23,7 +29,7 @@ from tsfpga.module import get_modules
 TSFPGA_EXAMPLES_TEMP_DIR = tsfpga.TSFPGA_GENERATED
 
 
-def get_default_registers():
+def get_default_registers() -> list[Register]:
     """
     Default registers for tsfpga examples.
     """
@@ -55,7 +61,9 @@ def get_default_registers():
     return registers
 
 
-def get_tsfpga_example_modules(names_include=None, names_avoid=None):
+def get_tsfpga_example_modules(
+    names_include: Optional[set[str]] = None, names_avoid: Optional[set[str]] = None
+) -> "ModuleList":
     """
     Wrapper of the regular :func:`.get_modules`. call with correct settings for tsfpga
     example modules.
@@ -72,12 +80,14 @@ def get_tsfpga_example_modules(names_include=None, names_avoid=None):
     )
 
 
-def get_hdl_modules(names_include=None, names_avoid=None):
+def get_hdl_modules(
+    names_include: Optional[set[str]] = None, names_avoid: Optional[set[str]] = None
+) -> "ModuleList":
     """
-    Wrapper of :func:`.get_modules` which returns the ``hdl_modules`` module objects
+    Wrapper of :func:`.get_modules` which returns the ``hdl-modules`` module objects
     (https://hdl-modules.com), if available.
 
-    If ``hdl_modules`` can not be found in the default repo checkout location,
+    If ``hdl-modules`` can not be found in the default repo checkout location,
     the function will assert False.
 
     Arguments will be passed on to :func:`.get_modules`.
@@ -85,8 +95,8 @@ def get_hdl_modules(names_include=None, names_avoid=None):
     Return:
         :class:`.ModuleList`: The module objects.
     """
-    # Presumed location of the hdl_modules repo
-    hdl_modules_repo_root = tsfpga.REPO_ROOT.parent.parent.resolve() / "hdl_modules" / "hdl_modules"
+    # Presumed location of the hdl-modules repo
+    hdl_modules_repo_root = tsfpga.REPO_ROOT.parent.parent.resolve() / "hdl-modules" / "hdl-modules"
     if (hdl_modules_repo_root / "modules").exists():
         return get_modules(
             modules_folders=[hdl_modules_repo_root / "modules"],
@@ -96,5 +106,5 @@ def get_hdl_modules(names_include=None, names_avoid=None):
         )
 
     raise FileNotFoundError(
-        f"The hdl_modules modules could not be found. Searched in {hdl_modules_repo_root}"
+        f"The hdl-modules modules could not be found. Searched in {hdl_modules_repo_root}"
     )
