@@ -3,27 +3,28 @@
 #
 # This file is part of the tsfpga project, a project platform for modern FPGA development.
 # https://tsfpga.com
-# https://gitlab.com/tsfpga/tsfpga
+# https://github.com/tsfpga/tsfpga
 # --------------------------------------------------------------------------------------------------
 
 # Standard libraries
 import os
 from pathlib import Path
+from typing import Any, Iterator, Optional, Union
 
 # First party libraries
 from tsfpga.system_utils import file_is_in_directory
 
 
-def get_git_commit(directory):
+def get_git_commit(directory: Path) -> str:
     """
     Get a string describing the current git commit.
     E.g. ``"abcdef0123"`` or ``"12345678 (local changes present)"``.
 
     Arguments:
-        directory (pathlib.Path): The directory where git commands will be run.
+        directory: The directory where git commands will be run.
 
-    Returns:
-        str: Git commit information.
+    Return:
+        Git commit information.
     """
     git_commit = get_git_sha(directory=directory)
     if git_local_changes_present(directory=directory):
@@ -32,15 +33,15 @@ def get_git_commit(directory):
     return git_commit
 
 
-def get_git_sha(directory):
+def get_git_sha(directory: Path) -> str:
     """
     Get a short git SHA.
 
     Arguments:
-        directory (pathlib.Path): The directory where git commands will be run.
+        directory: The directory where git commands will be run.
 
-    Returns:
-        str: The SHA.
+    Return:
+        The SHA.
     """
     # Generally, eight to ten characters are more than enough to be unique within a project.
     # The linux kernel, one of the largest projects, needs 11.
@@ -54,7 +55,7 @@ def get_git_sha(directory):
     # This function should only be called if git is available.
     # pylint: disable=import-outside-toplevel
     # Third party libraries
-    from git import Repo
+    from git.repo import Repo
 
     repo = Repo(directory, search_parent_directories=True)
     git_sha = repo.head.commit.hexsha[0:sha_length]
@@ -62,35 +63,36 @@ def get_git_sha(directory):
     return git_sha
 
 
-def git_local_changes_present(directory):
+def git_local_changes_present(directory: Path) -> bool:
     """
     Check if the git repo has local changes.
 
     Arguments:
-        directory (pathlib.Path): The directory where git commands will be run.
+        directory: The directory where git commands will be run.
 
-    Returns:
-        bool: ``True`` if the repo contains changes that have been made after the last commit.
+    Return:
+        ``True`` if the repo contains changes that have been made after the last commit.
     """
     # Import fails if "git" executable is not available, hence it can not be on top level.
     # This function should only be called if git is available.
     # pylint: disable=import-outside-toplevel
     # Third party libraries
-    from git import Repo
+    from git.repo import Repo
 
     repo = Repo(directory, search_parent_directories=True)
 
     return repo.is_dirty()
 
 
-def git_commands_are_available(directory):
+def git_commands_are_available(directory: Path) -> bool:
     """
     True if "git" command executable is available, and ``directory`` is in a valid git repo.
     """
     try:
         # pylint: disable=import-outside-toplevel
         # Third party libraries
-        from git import InvalidGitRepositoryError, Repo
+        from git import InvalidGitRepositoryError
+        from git.repo import Repo
     except ImportError:
         return False
 
@@ -103,30 +105,29 @@ def git_commands_are_available(directory):
 
 
 def find_git_files(
-    directory,
-    exclude_directories=None,
-    file_endings_include=None,
-    file_endings_avoid=None,
-):
+    directory: Path,
+    exclude_directories: Optional[list[Path]] = None,
+    file_endings_include: Optional[Union[str, tuple[str]]] = None,
+    file_endings_avoid: Optional[Union[str, tuple[str]]] = None,
+) -> Iterator[Path]:
     """
     Find files that are checked in to git.
 
     Arguments:
-        directory (pathlib.Path): Search in this directory.
-        exclude_directories (list(pathlib.Path)): Files in these directories will not be included.
-        file_endings_include (str or tuple(str)). Only files with these endings will be included.
-        file_endings_avoid (str or tuple(str)): String or tuple of strings. Files with these endings
-            will not be included.
+        directory: Search in this directory.
+        exclude_directories: Files in these directories will not be included.
+        file_endings_include: Only files with these endings will be included.
+        file_endings_avoid: Files with these endings will not be included.
 
-    Returns:
-        list(pathlib.Path): The files that are available in git.
+    Return:
+        The files that are available in git.
     """
 
     # Import fails if "git" executable is not available, hence it can not be on top level.
     # This function should only be called if git is available.
     # pylint: disable=import-outside-toplevel
     # Third party libraries
-    from git import Repo
+    from git.repo import Repo
 
     exclude_directories = (
         []
@@ -134,7 +135,7 @@ def find_git_files(
         else [exclude_directory.resolve() for exclude_directory in exclude_directories]
     )
 
-    def list_paths(root_tree, path):
+    def list_paths(root_tree: Any, path: Path) -> Iterator[Path]:
         for blob in root_tree.blobs:
             yield path / blob.name
         for tree in root_tree.trees:
