@@ -3,21 +3,21 @@
 #
 # This file is part of the tsfpga project, a project platform for modern FPGA development.
 # https://tsfpga.com
-# https://gitlab.com/tsfpga/tsfpga
+# https://github.com/tsfpga/tsfpga
 # --------------------------------------------------------------------------------------------------
 
 # Standard libraries
 import json
+from typing import Optional, Union
 
 # Local folder libraries
 from .logic_level_distribution_parser import LogicLevelDistributionParser
 
 
 class BuildResult:
-
     """
     Attributes:
-        project_name (`str`): The name of the build.
+        name (`str`): The name of the build.
         success (`bool`): True if the build and all pre- and post hooks succeeded.
         synthesis_size (`dict`): A dictionary with the utilization of primitives for the
             synthesized design.
@@ -30,64 +30,63 @@ class BuildResult:
             Will be ``None`` if synthesis failed or did not run.
     """
 
-    def __init__(self, name):
+    def __init__(self, name: str):
         """
         Arguments:
-            name (`str`): The name of the build.
+            name: The name of the build.
         """
         self.name = name
-        self.success = True
-        self.synthesis_size = None
-        self.implementation_size = None
-        self.logic_level_distribution = None
+        self.success: bool = True
+        self.synthesis_size: Optional[dict[str, int]] = None
+        self.implementation_size: Optional[dict[str, int]] = None
+        self.logic_level_distribution: Optional[str] = None
 
-    def size_summary(self):
+    def size_summary(self) -> Union[str, None]:
         """
         Return a string with a formatted message of the size.
 
-        Returns:
-            str: A human-readable message of the latest size. ``None`` if no size is set.
+        Return:
+            A human-readable message of the latest size. ``None`` if no size is set.
         """
         build_step = None
         size = None
 
-        if self.implementation_size is not None:
+        if self.implementation_size:
             build_step = "implementation"
             size = self.implementation_size
-        elif self.synthesis_size is not None:
+
+        elif self.synthesis_size:
             build_step = "synthesis"
             size = self.synthesis_size
+
         else:
             return None
 
         return f"Size of {self.name} after {build_step}:\n{json.dumps(size, indent=2)}"
 
-    def report(self):
+    def report(self) -> Union[str, None]:
         """
         Return a report of the build result. Includes all metrics and information that has been
         extracted from the Vivado reports.
-
-        Returns:
-            str: The report.
         """
         result = self.size_summary()
 
-        if self.logic_level_distribution is not None:
+        if self.logic_level_distribution:
             result = f"{result}\nLogic level distribution:\n{self.logic_level_distribution}"
 
         return result
 
     @property
-    def maximum_logic_level(self):
+    def maximum_logic_level(self) -> Union[None, int]:
         """
         The maximum level in the the :attr:`.BuildResult.logic_level_distribution`.
         Will be ``None`` for non-netlist builds.
         Will be ``None`` if synthesis failed or did not run.
 
-        Returns:
-            int: The maximum logic level.
+        Return:
+            The maximum logic level.
         """
-        if self.logic_level_distribution is None:
+        if not self.logic_level_distribution:
             return None
 
         return LogicLevelDistributionParser.get_maximum_logic_level(
